@@ -1,28 +1,29 @@
 
+import json
 import logging
+import operator
 from pathlib import Path
 from dataclasses import dataclass
-import sys
-from typing import TYPE_CHECKING, Callable, Union
-from unittest.mock import patch
+from typing import TYPE_CHECKING, Any, Callable, Optional, Union
+
 from PyQt5.QtGui import QCursor, QGuiApplication
 from PyQt5.QtWidgets import QMainWindow, QMessageBox, QWidget
 from PyQt5.QtCore import QTimer, QSettings, QThread, QTranslator, QLocale
 
-from .patchcanvas import patchcanvas, PortType
+from .patchcanvas import patchcanvas, PortType, PortSubType, PortMode
 from .patchcanvas.utils import get_new_group_positions
 from .patchcanvas.scene_view import PatchGraphicsView
 from .patchcanvas.init_values import (
-    CallbackAct, CanvasFeaturesObject, CanvasOptionsObject, PortSubType)
-from .patchcanvas.theme_manager import ThemeManager
+    CallbackAct, CanvasFeaturesObject, CanvasOptionsObject)
 
 from .patchbay_signals import SignalsObject
 from .tools_widgets import PatchbayToolsWidget
 from .canvas_menu import CanvasMenu
 from .options_dialog import CanvasOptionsDialog
 from .filter_frame import FilterFrame
-from .base_elements import (Connection, GroupPos, Port, PortTypesViewFlag, Portgroup, Group,
-                            JackPortFlag, PortgroupMem, ToolDisplayed, TransportPosition)
+from .base_elements import (
+    Connection, GroupPos, Port, PortTypesViewFlag, Portgroup, Group,
+    JackPortFlag, PortgroupMem, ToolDisplayed, TransportPosition)
 from .calbacker import Callbacker
 
 
@@ -125,6 +126,8 @@ class PatchbayManager:
         self.options_dialog: CanvasOptionsDialog = None
         self.filter_frame: FilterFrame = None
         
+        self._manual_path: Path = None
+
         self.sg = SignalsObject()
 
         self._next_group_id = 0
@@ -172,6 +175,7 @@ class PatchbayManager:
     def app_init(self,
                  view: PatchGraphicsView,
                  theme_paths: list[Path],
+                 manual_path: Path = None,
                  callbacker: Callbacker = None,
                  options: CanvasOptionsObject = None,
                  features: CanvasFeaturesObject = None,
@@ -184,7 +188,7 @@ class PatchbayManager:
             self.callbacker = callbacker
             self.sg.callback_sig.connect(self.callbacker.receive)
         
-        # theme_paths = ThemeManager.default_theme_paths(source_theme_path)
+        self._manual_path = manual_path
         
         if options is None:
             options = patchcanvas.CanvasOptionsObject()
@@ -215,6 +219,10 @@ class PatchbayManager:
         patchcanvas.set_options(options)
         patchcanvas.set_features(features)
         patchcanvas.init(view, self.__canvas_callback__, theme_paths, default_theme_name)
+        patchcanvas.canvas.scene.scale_changed.connect(self._scene_scale_changed)
+        
+    def _scene_scale_changed(self, value: float):
+        self.sg.scene_scale_changed.emit(value)
 
     def _execute_in_main_thread(self, func: Callable, args: tuple, kwargs: dict):
         func(*args, **kwargs)
@@ -319,7 +327,7 @@ class PatchbayManager:
 
     def get_port_from_id(self, group_id: int, port_id: int) -> Port:
         group = self.get_group_from_id(group_id)
-        if group is not None:        
+        if group is not None:
             for port in group.ports:
                 if port.port_id == port_id:
                     return port
@@ -506,6 +514,15 @@ class PatchbayManager:
         self.optimize_operation(False)
         patchcanvas.redraw_all_groups()
 
+    def set_zoom(self, zoom: float):
+        patchcanvas.canvas.scene.zoom_ratio(zoom)
+
+    def zoom_reset(self):
+        patchcanvas.zoom_reset()
+    
+    def zoom_fit(self):
+        patchcanvas.zoom_fit()  
+
     def refresh(self):
         self.clear_all()
 
@@ -665,7 +682,7 @@ class PatchbayManager:
             return group.group_id
 
     @later_by_batch(sort_group=True)
-    def metadata_update(self, uuid: int, key: str, value: str) -> int:
+    def metadata_update(self, uuid: int, key: str, value: str) -> Optional[int]:
         ''' remember metadata and returns the group_id'''
         if key == JACK_METADATA_ORDER:
             port = self.get_port_from_uuid(uuid)
@@ -706,6 +723,21 @@ class PatchbayManager:
                     group.set_client_icon(value, from_metadata=True)
                     return group.group_id
 
+        elif key == JACK_METADATA_SIGNAL_TYPE:
+            port = self.get_port_from_uuid(uuid)
+            if port is None:
+                return
+
+            port.mdata_signal_type = value
+
+            if port.type is PortType.AUDIO_JACK:
+                if value == 'CV':
+                    port.subtype = PortSubType.CV
+                elif value == 'AUDIO':
+                    port.subtype = PortSubType.REGULAR
+
+            return port.group_id
+
     @later_by_batch()
     def add_connection(self, port_out_name: str, port_in_name: str):
         port_out = self.get_port_from_name(port_out_name)
@@ -916,3 +948,146 @@ class PatchbayManager:
 
         if some_groups_removed:
             patchcanvas.canvas.scene.resize_the_scene()
+
+    def _export_port_list_to_patchichi(self) -> str:
+        def slcol(input_str: str) -> str:
+            return input_str.replace(':', '\\:')
+        
+        contents = ''
+
+        gps_and_ports = list[tuple[str, list[Port]]]()
+        for group in self.groups:
+            for port in group.ports:
+                group_name = port.full_name.partition(':')[0]
+                for gp_name, gp_port_list in gps_and_ports:
+                    if gp_name == group_name:
+                        gp_port_list.append(port)
+                        break
+                else:
+                    gps_and_ports.append((group_name, [port]))
+
+        for group_name, port_list in gps_and_ports:
+            port_list.sort(key=operator.attrgetter('port_id'))
+                
+        for group_name, port_list in gps_and_ports:
+            gp_written = False              
+            last_type_and_mode = (PortType.NULL, PortMode.NULL)
+            physical = False
+            terminal = False
+            monitor = False
+            pg_name = ''
+            signal_type = ''
+
+            for port in port_list:
+                if not gp_written:
+                    contents += f'\n::{group_name}\n'
+                    gp_written = True
+
+                    group = self.get_group_from_name(group_name)
+                    if group is not None:
+                        group_attrs = list[str]()
+                        if group.client_icon:
+                            group_attrs.append(f'CLIENT_ICON={slcol(group.client_icon)}')
+                            
+                        if group.mdata_icon:
+                            group_attrs.append(f'ICON_NAME={slcol(group.mdata_icon)}')
+
+                        if group.has_gui:
+                            if group.gui_visible:
+                                group_attrs.append('GUI_VISIBLE')
+                            else:
+                                group_attrs.append('GUI_HIDDEN')
+                        if group_attrs:
+                            contents += ':'
+                            contents += '\n:'.join(group_attrs)
+                            contents += '\n'
+
+                if port.flags & JackPortFlag.IS_PHYSICAL:
+                    if not physical:
+                        contents += ':PHYSICAL\n'
+                        physical = True
+                elif physical:
+                    contents += ':~PHYSICAL\n'
+                    physical = False
+
+                if last_type_and_mode != (port.type, port.mode()):
+                    if port.type is PortType.AUDIO_JACK:
+                        if port.flags & JackPortFlag.IS_CONTROL_VOLTAGE:
+                            contents += ':CV'
+                        else:
+                            contents += ':AUDIO'
+                    elif port.type is PortType.MIDI_JACK:
+                        contents += ':MIDI'
+
+                    contents += f':{port.mode().name}\n'
+                    last_type_and_mode = (port.type, port.mode())
+                
+                if port.mdata_signal_type != signal_type:
+                    if port.mdata_signal_type:
+                        contents += f':SIGNAL_TYPE={slcol(port.mdata_signal_type)}\n'
+                    else:
+                        contents += ':~SIGNAL_TYPE\n'
+                
+                if port.mdata_portgroup != pg_name:
+                    if port.mdata_portgroup:
+                        contents += f':PORTGROUP={slcol(port.mdata_portgroup)}\n'
+                    else:
+                        contents += ':~PORTGROUP\n'
+                    pg_name = port.mdata_portgroup
+
+                port_short_name = port.full_name.partition(':')[2]
+                contents += f'{port_short_name}\n'
+                
+                if port.pretty_name or port.order:
+                    port_attrs = list[str]()
+                    if port.pretty_name:
+                        port_attrs.append(f'PRETTY_NAME={slcol(port.pretty_name)}')
+                    if port.order:
+                        port_attrs.append(f'ORDER={port.order}')
+                    contents += ':'
+                    contents += '\n:'.join(port_attrs)
+                    contents += '\n'
+
+        return contents
+    
+    def export_to_patchichi_json(
+            self, path: Path, editor_text='') -> bool:
+        if not editor_text:
+            editor_text = self._export_port_list_to_patchichi()
+
+        file_dict = dict[str, Any]()
+        file_dict['VERSION'] = (0, 1)       
+        file_dict['editor_text'] = editor_text
+        file_dict['connections'] = [
+            (c.port_out.full_name, c.port_in.full_name)
+            for c in self.connections]
+        file_dict['group_positions'] = [
+            gpos.as_serializable_dict() for gpos in self.group_positions
+            if self.get_group_from_name(gpos.group_name) is not None]
+        
+        portgroups = list[dict[str, Any]]()
+        for pg_mem in self.portgroups_memory:
+            group = self.get_group_from_name(pg_mem.group_name)
+            if group is None:
+                continue
+
+            for port_str in pg_mem.port_names:
+                for port in group.ports:
+                    if (port.short_name() == port_str
+                            and port.type == pg_mem.port_type
+                            and port.mode() == pg_mem.port_mode):
+                        portgroups.append(pg_mem.as_serializable_dict())
+                        break
+        
+        file_dict['portgroups'] = portgroups
+        
+        try:
+            with open(path, 'w') as f:
+                json.dump(file_dict, f, indent=2)
+            return True
+        except Exception as e:
+            _logger.error(f'Failed to save patchichi file: {str(e)}')
+            return False
+        
+            
+        
\ No newline at end of file
