@@ -20,7 +20,7 @@
 # global imports
 import logging
 from pathlib import Path
-from typing import Callable
+from typing import Callable, Optional
 from PyQt5.QtCore import (pyqtSlot, QObject, QPoint, QPointF, QRectF,
                           QSettings, QTimer, pyqtSignal)
 from PyQt5.QtWidgets import QAction, QMainWindow
@@ -82,11 +82,10 @@ class CanvasObject(QObject):
     connection_added = pyqtSignal(int)
     connection_removed = pyqtSignal(int)
     move_boxes_finished = pyqtSignal()
-    zoom_changed = pyqtSignal(int)
 
     def __init__(self, parent=None):
         QObject.__init__(self, parent)
-        self.groups_to_join = []
+        self.groups_to_join = list[int]()
         self.move_boxes_finished.connect(self.join_after_move)
 
     @pyqtSlot()
@@ -234,7 +233,7 @@ def set_canvas_size(x: int, y: int, width: int, height: int):
 @patchbay_api
 def set_loading_items(yesno: bool):
     '''while canvas is loading items (groups or ports, connections...)
-    then, items will be added, but no redrawn.
+    then, items will be added, but not redrawn.
     This is an optimization that prevents a lot of redraws.
     Think to set loading items at False and use redraw_all_groups
     or redraw_group once the long operation is finished'''
@@ -258,7 +257,7 @@ def add_group(group_id: int, group_name: str, split=BoxSplitMode.UNDEF,
     group.group_id = group_id
     group.group_name = group_name
     group.split = bool(split == BoxSplitMode.YES)
-    group.icon_type = icon_type
+    group.box_type = icon_type
     group.icon_name = icon_name
     group.layout_modes = layout_modes
     group.plugin_id = -1
@@ -459,7 +458,7 @@ def split_group(group_id: int, on_place=False):
 
     # Step 3 - Re-create Item, now split
     add_group(group_id, g.group_name, BoxSplitMode.YES,
-              g.icon_type, g.icon_name, g.layout_modes,
+              g.box_type, g.icon_name, g.layout_modes,
               null_xy=(g.null_pos.x(), g.null_pos.y()),
               in_xy=(g.in_pos.x(), g.in_pos.y()),
               out_xy=(g.out_pos.x(), g.out_pos.y()),
@@ -542,7 +541,7 @@ def join_group(group_id: int):
     g = tmp_group
     # Step 3 - Re-create Item, now together
     add_group(group_id, g.group_name, BoxSplitMode.NO,
-              g.icon_type, g.icon_name, g.layout_modes,
+              g.box_type, g.icon_name, g.layout_modes,
               null_xy=(g.null_pos.x(), g.null_pos.y()),
               in_xy=(g.in_pos.x(), g.in_pos.y()),
               out_xy=(g.out_pos.x(), g.out_pos.y()))
@@ -775,16 +774,18 @@ def set_group_pos_full(group_id, group_pos_x_o, group_pos_y_o,
     
 
 @patchbay_api
-def set_group_icon(group_id: int, icon_type: BoxType, icon_name: str):
+def set_group_icon(group_id: int, box_type: BoxType, icon_name: str):
     group = canvas.get_group(group_id)
     if group is None:
         _logger.critical(f"{_logging_str} - unable to find group to change icon")
         return
     
-    group.icon_type = icon_type
+    group.box_type = box_type
+    group.icon_name = icon_name
+
     for widget in group.widgets:
         if widget is not None:
-            widget.set_icon(icon_type, icon_name)
+            widget.set_icon(box_type, icon_name)
 
     QTimer.singleShot(0, canvas.scene.update)
 
@@ -1223,6 +1224,25 @@ def set_connection_in_front(connection_id: int):
     if connection and connection.widget is not None:
         connection.widget.setZValue(canvas.last_z_value)
 
+@patchbay_api
+def select_port(group_id: int, port_id: int):
+    port = canvas.get_port(group_id, port_id)
+    if port is None:
+        return
+    
+    if port.widget is None:
+        return
+    
+    box_widget = port.widget.parentItem()
+    canvas.scene.clearSelection()
+
+    if box_widget.is_wrapped():
+        canvas.scene.center_view_on(box_widget)
+        box_widget.setSelected(True)
+    else:
+        canvas.scene.center_view_on(port.widget)
+        port.widget.setSelected(True)
+
 @patchbay_api
 def select_filtered_group_box(group_id: int, n_select=1):
     group = canvas.get_group(group_id)
